//  ______ __  __  ____       _ _____    _____ _    _          _____  ______ _____  
// |  ____|  \/  |/ __ \     | |_   _|  / ____| |  | |   /\   |  __ \|  ____|  __ \ 
// | |__  | \  / | |  | |    | | | |   | (___ | |__| |  /  \  | |  | | |__  | |__) |
// |  __| | |\/| | |  | |_   | | | |    \___ \|  __  | / /\ \ | |  | |  __| |  _  / 
// | |____| |  | | |__| | |__| |_| |_   ____) | |  | |/ ____ \| |__| | |____| | \ \ 
// |______|_|  |_|\____/ \____/|_____| |_____/|_|  |_/_/    \_\_____/|______|_|  \_\
// | |           | ____|                | |    | |                                  
// | |__  _   _  | |____      _____  ___| |_ __| | _____   __                       
// | '_ \| | | | |___ \ \ /\ / / _ \/ _ \ __/ _` |/ _ \ \ / /                       
// | |_) | |_| |  ___) \ V  V /  __/  __/ || (_| |  __/\ V /                        
// |_.__/ \__, | |____/ \_/\_/ \___|\___|\__\__,_|\___| \_/                         
//         __/ |                                                                    
//        |___/                                                                     
//
//                                        
// Feel free to change shader parameters

#version 130
#extension GL_ARB_explicit_attrib_location : enable

uniform float viewWidth;
uniform float viewHeight;

uniform sampler2D colortex0;
uniform sampler2D colortex4;
uniform sampler2D colortex5;
uniform sampler2D colortex3;
uniform sampler2D depthtex0;

uniform float far;
uniform float near;

const float edgeThreshold = 0.05;

#include "/lib/settings.glsl"

in vec2 texcoord;

/* DRAWBUFFERS:03 */
out vec4 colortex0Out;
out vec4 colortex3Out;


vec3 quantizeColor(vec3 color, float steps) {
    // Ensure steps is at least 2 to avoid division by zero
    steps = max(steps, 2.0);
    
    // Calculate the step size
    float stepSize = 1.0 / (steps - 1.0);
    
    // Quantize each channel
    vec3 quantized;
    quantized.r = round(color.r / stepSize) * stepSize;
    quantized.g = round(color.g / stepSize) * stepSize;
    quantized.b = round(color.b / stepSize) * stepSize;
    
    // Clamp the result to [0, 1] range
    return clamp(quantized, 0.0, 1.0);
}

vec2 getMapping(int index) {
    int x = index % 16;
    int y = index / 16;
    if (index == 0){
        return vec2(0.0, 6.0);
    } 
    return vec2(float(x), float(y));
}

// Tone mapping function
vec3 toneMap(vec3 color) {
    return color / (vec3(1.0) + color);
}

vec3 acesTonemap(vec3 color) {
    float a = 2.51;
    float b = 0.03;
    float c = 2.43;
    float d = 0.59;
    float e = 0.14;
    return clamp((color * (a * color + b)) / (color * (c * color + d) + e), 0.0, 1.0);
}

void main() {
	vec2 texelSize = vec2(1.0 / viewWidth, 1.0 / viewHeight);
	vec2 chars          = vec2(viewWidth  / FONT_WIDTH        , viewHeight  / FONT_HEIGHT);
	vec2 nCharSize      = vec2(FONT_WIDTH / FONT_TEXTURE_WIDTH, FONT_HEIGHT / FONT_TEXTURE_HEIGHT); 
	vec2 charsInTexture = vec2(FONT_TEXTURE_WIDTH / FONT_WIDTH, FONT_TEXTURE_HEIGHT / FONT_HEIGHT);

	// downscale
    vec2 foo = (texcoord * chars);
	vec2 downscaledCoord = vec2(floor(foo.x), round(foo.y)) / chars;

#ifdef MORE_FPS
    vec3 avarageColor = texture(colortex0, downscaledCoord).rgb;
#else
    // avarage
	vec3 avarageColor = vec3(0.0);
	for(float i = 0.0; i < FONT_WIDTH; i += 1.0) {
		for(float j = 0.0; j < FONT_HEIGHT; j += 1.0) {
			avarageColor = avarageColor + texture(colortex0, downscaledCoord + vec2(texelSize.x * i, texelSize.y * j)).rgb;
		}
	}
	avarageColor = avarageColor / (FONT_WIDTH * FONT_HEIGHT);
#endif

    //avarageColor = acesTonemap(avarageColor);

	vec3 color = texture(colortex0, downscaledCoord).rgb;

	// Gray scale
	float intensity = dot(avarageColor.rgb, vec3(0.299, 0.587, 0.114)); 

    vec3 charColor;
    vec2 charCoord = mod(mod((vec2(texcoord.x, 1.0 - texcoord.y - (1.0 / viewWidth))) * vec2(chars.x, chars.y), 1.0) / charsInTexture, 1.0);
    

    // Find best color matching character index
    float bestDistance = 1.0e10;
    int bestIndex = -1;
    for(int i = 0; i < int(FONT_NUMBER_OF_CHARS); i += 1)
    {
        vec2 index = getMapping(i) * nCharSize;
        vec3 currentColor = texture(colortex5, index).rgb;

        float dist = length(avarageColor - currentColor);

        if(dist < bestDistance){
            bestDistance = dist;
            bestIndex = i;
        }
    }

    // Get character index based on intensity
    int intensityIndex = int(floor(intensity * FONT_NUMBER_OF_CHARS));

    // Mix intesity and color matching characters
    vec2 finalIndex = getMapping(int(mix(float(intensityIndex), float(bestIndex), COLOR_INTENSITY_MATCHING))) * nCharSize;

    charColor = texture(colortex5, charCoord + finalIndex).rgb;

    // Hide top left pixel that represents color in the texture
    if(charCoord.x <= 0.01 && charCoord.y <= 0.01)
    {
        charColor = vec3(0.0);
    }


    #ifdef COLOR
        #ifdef QUANTIZE_COLOR
	colortex0Out = vec4(charColor * quantizeColor(color * 2.5, QUANTIZE_COLOR_VALUE), 1.0);
        #else
    colortex0Out = vec4(charColor * color, 1.0);
        #endif
    #else
    colortex0Out = vec4(charColor, 1.0);
    #endif

    colortex3Out = vec4(texture(colortex0, texcoord).rgb, 1.0);
}